// Proelec remote control socket controller
// David Pilling March 2022


#include <SerialCommand.h>
SerialCommand sCmd;


const int T=176;
const int transmit_pin=5;
const int nrepeats=6;

void pr(char *fmt, ... )
{
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.println(buf);
 
}


void high(void)
{
 digitalWrite(transmit_pin, HIGH);
}

void low(void)
{
 digitalWrite(transmit_pin, LOW);
}

void shortpulse(void)
{
 high();
 delayMicroseconds(T);
 low();
 delayMicroseconds(3*T); 
}

void longpulse(void)
{
 high();
 delayMicroseconds(3*T);
 low();
 delayMicroseconds(T); 
}

void syncpulse(void)
{
 high();
 delayMicroseconds(T);
 low();
 delayMicroseconds(31*T); 
}

void floatpulse(void)
{
 shortpulse();
 longpulse();
}

void zeropulse(void)
{
 shortpulse();
 shortpulse();
}

void onepulse(void)
{
 longpulse();
 longpulse();
}

void sendswitchlow(char * arg)
{
 char * p=arg;
 char   c;
 while((c=*p++))
 {
  switch(c)
  {
   case '0':zeropulse();
            break;

   case '1':onepulse();
            break;
            
   case 'F':floatpulse();
            break;

   case 'S':syncpulse();
            break; 
  }
 }
}




void sendswitch(char * arg)
{
 int i;
 for(i=0;i<nrepeats;i++)
 {
  sendswitchlow(arg);
 }
}


char * table[3][2]=
{
 "FFF0FFFF0110S",
 "FFF0FFFF0101S",
 "FFF0FFFF1010S",
 "FFF0FFFF1001S",
 "FFF0FFF10010S",
 "FFF0FFF10001S"

};


void cmdswitch(void)
{ 
 char * arg; 
 int    unit;
 int    state;
 
 arg = sCmd.next();
 if(arg!=NULL)
 {
  unit=atoi(arg);
  arg = sCmd.next();
  if(arg!=NULL)
  {
   state=atoi(arg);
   if(unit>=1 && unit<=3 && (state==0 || state==1))
   {
    pr("Switch %d %d",unit,state);
    unit--;
    arg=table[unit][state];
    sendswitch(arg);
    pr(arg);
    return;
   }
  }
 }

 pr("Syntax: s <unit 1-3> <state 0|1>"); 
}


void setup() 
{
 Serial.begin(9600);        // connect to the serial port
 pinMode(transmit_pin,OUTPUT);
 low();
 sCmd.addCommand("s",cmdswitch);
 
}

void loop() 
{
if(Serial.available()>0) 
 {
  sCmd.readSerial();
 }

}
